==> cmsdkv3-3.0.0-sources.jar: net/consentmanager/cm_sdk_android_v3/CMPManager.kt <==
package net.consentmanager.cm_sdk_android_v3

import android.app.Activity
import android.content.Context
import android.util.Log
import android.webkit.WebView
import androidx.annotation.Keep
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.serialization.json.JsonObject
import java.lang.ref.WeakReference
import java.util.concurrent.atomic.AtomicReference

@Keep
class JavaFriendlyResult<T> internal constructor(private val kotlinResult: Result<T>) {
    fun isSuccess(): Boolean = kotlinResult.isSuccess
    fun exceptionOrNull(): Throwable? = kotlinResult.exceptionOrNull()
}

@Keep
class CMPManager private constructor(
    private val context: Context,
    private val urlConfig: UrlConfig,
    private val webViewConfig: ConsentLayerUIConfig,
    private val delegate: CMPManagerDelegate
) : WebViewManagerDelegate, NetworkMonitorDelegate,
    DefaultLifecycleObserver {

    companion object {
        private val instanceRef: AtomicReference<CMPManager?> = AtomicReference(null)

        fun getInstance(
            context: Context,
            urlConfig: UrlConfig,
            webViewConfig: ConsentLayerUIConfig = ConsentLayerUIConfig(),
            delegate: CMPManagerDelegate
        ): CMPManager {
            return instanceRef.get() ?: synchronized(this) {
                instanceRef.get() ?: CMPManager(
                    context.applicationContext,
                    urlConfig,
                    webViewConfig,
                    delegate
                ).also {
                    instanceRef.set(it)
                }
            }
        }
    }

    private val coroutineScope = CoroutineScope(Dispatchers.Main)
    private val userPreferences: CMPUserPreferencesService = CMPUserPreferencesServiceImpl(context)
    private val webViewManager: WebViewManager
    private val networkMonitor: NetworkMonitor = NetworkMonitor(context)
    private val retryManager: RetryManager

    private var activityContext: WeakReference<Activity>? = null

    private var consentLayerDialog: ConsentLayerDialog? = null
    private var webView: WebView? = null
    private var webViewContainer: WebViewContainer? = null

    init {
        networkMonitor.delegate = this

        webViewManager = WebViewManager(userPreferences, coroutineScope, networkMonitor)
        webViewManager.delegate = this

        retryManager = RetryManager(maxRetryCount = 3, baseDelay = 2000L, retryable = webViewManager)

        setupWebView()
    }

    fun setActivity(activity: Activity) {
        activityContext = WeakReference(activity)
        setupWebView()
    }

    fun setWebView(webView: WebView) {
        this.webView = webView
        setupWebView()
    }

    private fun setupWebView() {
        val activity = activityContext?.get() ?: return

        webView = webViewManager.createWebView(WeakReference(activity))
        webViewContainer = webView?.let { WebViewContainer(it) }
    }

    private fun showConsentLayer() {
        Log.d("CMPManager", "Showing consent layer")
        val activity = activityContext ?: run {
            delegate.didReceiveError("No valid activity context available")
            return
        }
        val config = webViewConfig

        if (webView == null) {
            webView = webViewManager.createWebView(activity)
            webViewContainer = WebViewContainer(webView!!)
        }

        if (consentLayerDialog == null || consentLayerDialog?.isShowing() == false) {
            Log.d("CMPManager", "Creating new consent layer dialog")
            consentLayerDialog?.dismiss()
            consentLayerDialog = ConsentLayerDialog(activity.get()!!, config, webViewContainer!!, webViewManager)
            consentLayerDialog?.show()
        }
        delegate.didShowConsentLayer()
    }

    private fun dismissConsentLayer() {
        Log.d("CMPManager", "Dismissing consent layer")

        consentLayerDialog?.dismiss()
        consentLayerDialog = null
    }

    private fun createUrlParams(useCase: UseCase): CMPUrlParams {
        return CMPUrlParams(
            id = urlConfig.id,
            domain = urlConfig.domain,
            useCase = useCase,
            consent = getConsentModel()?.exportCMPInfo(),
            language = urlConfig.language,
            appName = urlConfig.appName,
            packageName = context.packageName,
            isDebugMode = false
        )
    }

    private fun getConsentModel(): CMPConsentModel? {
        val (jsonString, _) = userPreferences.getConsentData()
        return jsonString?.let { CMPConsentModel.fromJson(it) }
    }

    // Methods that do not use the WebView
    fun hasUserChoice(): Boolean = getConsentModel()?.hasUserChoice() ?: false
    fun hasPurposeConsent(id: String): Boolean = getConsentModel()?.hasPurposeConsent(id) ?: false
    fun hasVendorConsent(id: String): Boolean = getConsentModel()?.hasVendorConsent(id) ?: false
    fun exportCMPInfo(): String = getConsentModel()?.exportCMPInfo() ?: ""
    fun getAllPurposesIDs(): List<String> = getConsentModel()?.getAllPurposesIDs() ?: emptyList()
    fun getEnabledPurposesIDs(): List<String> = getConsentModel()?.getEnabledPurposesIDs() ?: emptyList()
    fun getDisabledPurposesIDs(): List<String> = getConsentModel()?.getDisabledPurposesIDs() ?: emptyList()
    fun getAllVendorsIDs(): List<String> = getConsentModel()?.getAllVendorsIDs() ?: emptyList()
    fun getEnabledVendorsIDs(): List<String> = getConsentModel()?.getEnabledVendorsIDs() ?: emptyList()
    fun getDisabledVendorsIDs(): List<String> = getConsentModel()?.getDisabledVendorsIDs() ?: emptyList()
    fun resetConsentManagementData(): Unit = userPreferences.resetConsentManagementData()

    fun checkWithServerAndOpenIfNecessary(completion: (Result<Unit>) -> Unit) {
        if (activityContext?.get() == null) {
            completion(Result.failure(IllegalStateException("Activity reference is null. Call setActivity() before checking consent.")))
            return
        }

        loadConsentURLWithUseCase(UseCase.VERIFY_CONSENT_ON_INITIALIZE) { result ->
            result.onSuccess { needsConsent ->
                if (needsConsent) {
                    showConsentLayer()
                } else {
                    delegate.didCloseConsentLayer()
                }
                completion(Result.success(Unit))
            }.onFailure { error ->
                completion(Result.failure(error))
            }
        }
    }

    fun openConsentLayer(completion: (Result<Unit>) -> Unit) {
        Log.d("CMPManager", "Force opening consent layer")

        webView = activityContext?.let { webViewManager.createWebView(it) }
        webViewContainer = WebViewContainer(webView!!)

        loadConsentURLWithUseCase(UseCase.OPEN_CONSENT) { result ->
            result.onSuccess {
                showConsentLayer()
                delegate.didShowConsentLayer()
                completion(Result.success(Unit))
            }.onFailure { error ->
                Log.e("CMPManager", "Error in forceOpenConsentLayer: ${error.message}")
                completion(Result.failure(error))
            }
        }
    }

    fun checkIfConsentIsRequired(completion: (Boolean) -> Unit) {
        loadConsentURLWithUseCase(UseCase.PERFORM_DRY_CHECK_CONSENT) { result ->
            result.onSuccess { needsConsent ->
                completion(needsConsent)
            }.onFailure {
                completion(true) // Assume consent is needed if there's an error
            }
        }
    }

    fun acceptVendors(vendors: List<String>, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.ENABLE_CONSENT_VENDORS, mapOf("addVendors" to vendors)) { result ->
            completion(result.map {  })
        }
    }

    fun rejectVendors(vendors: List<String>, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.DISABLE_CONSENT_VENDORS, mapOf("addVendors" to vendors)) { result ->
            completion(result.map {  })
        }
    }

    fun acceptPurposes(purposes: List<String>, updatePurpose: Boolean, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.ENABLE_CONSENT_PURPOSES, mapOf("addPurposes" to purposes, "updateVendors" to updatePurpose)) { result ->
            completion(result.map {  })
        }
    }

    fun rejectPurposes(purposes: List<String>, updateVendor: Boolean, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.DISABLE_CONSENT_PURPOSES, mapOf("addPurposes" to purposes, "updateVendors" to updateVendor)) { result ->
            completion(result.map {  })
        }
    }

    fun rejectAll(completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.REJECT_ALL_CONSENT, mapOf("rejectAll" to true)) { result ->
            completion(result.map {  })
        }
    }

    fun acceptAll(completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.ACCEPT_ALL_CONSENT, mapOf("acceptAll" to true)) { result ->
            completion(result.map {  })
        }
    }

    private fun loadConsentURLWithUseCase(useCase: UseCase, additionalParams: Map<String, Any>? = null, completion: (Result<Boolean>) -> Unit) {
        val params = createUrlParams(useCase)
        additionalParams?.let { params.apply(it) }

        val urlString = CMPUrlBuilder.build(params)
        println("Debug: URL for $useCase - $urlString") // Add this line for debugging

        webViewManager.loadConsentURL(urlString, useCase) { result ->
            completion(result)
        }
    }

    fun onApplicationPause() {
        webViewManager.cancelOperations()
        Log.d("CMPManager", "Application paused")
    }

    fun onApplicationResume() {
        Log.d("CMPManager", "Application resumed")
    }

    override fun didReceiveConsentMessage(consent: String, jsonObject: JsonObject) {
        Log.d("CMPManager", "Consent message received")
        delegate.didReceiveConsent(consent, jsonObject)
        dismissConsentLayer()
    }

    override fun didReceiveOpenMessage() {
        Log.d("CMPManager", "Open message received")
        coroutineScope.launch(Dispatchers.Main) {
            showConsentLayer()
        }
        delegate.didShowConsentLayer()
    }

    override fun didNotNeedOpenConsentLayer() {
        Log.d("CMPManager", "Close message received")
    }

    override fun didReceiveError(error: String) {
        Log.e("CMPManager", "Error received: $error")
        delegate.didReceiveError(error)
    }

    override fun networkStatusDidChange(isConnected: Boolean) {
        if (isConnected) {
            // Retry any pending operations if the network is back
            retryManager.retry { /* Handle retry result if needed */ }
        }
    }

    override fun onDestroy(owner: LifecycleOwner) {
        Log.d("CMPManager", "Activity is being destroyed")
        if (owner.lifecycle.currentState.isAtLeast(Lifecycle.State.RESUMED)) {
            Log.d("CMPManager", "Configuration is changing, not cleaning up")
            return
        }
        cleanUp()
    }

    private fun cleanUp() {
        activityContext?.clear()
        dismissConsentLayer()
        webViewContainer = null
    }

    override fun onResume(owner: LifecycleOwner) {
        Log.d("CMPManager", "Activity resumed")
        // Reinitialize WebView if needed
        val activity = activityContext
        if (activity != null && webViewManager.isWebViewDestroyed()) {
            val webView = webViewManager.createWebView(activity)
            webViewContainer = webView?.let { WebViewContainer(it) }
        }
    }

    fun onActivityDestroyed() {
        activityContext = null
        dismissConsentLayer()
        webView = null
        webViewContainer = null
    }
}

==> cmsdkv3-3.1.0-sources.jar: net/consentmanager/cm_sdk_android_v3/CMPManager.kt <==
package net.consentmanager.cm_sdk_android_v3

import android.app.Activity
import android.content.Context
import android.util.Log
import android.webkit.WebView
import androidx.annotation.Keep
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.suspendCancellableCoroutine
import kotlinx.serialization.json.JsonObject
import net.consentmanager.cm_sdk_android_v3.CMPConsentModel.Companion.toUserConsentStatus
import java.lang.ref.WeakReference
import java.util.concurrent.atomic.AtomicReference

@Keep
class JavaFriendlyResult<T> internal constructor(private val kotlinResult: Result<T>) {
    fun isSuccess(): Boolean = kotlinResult.isSuccess
    fun exceptionOrNull(): Throwable? = kotlinResult.exceptionOrNull()
}

@Keep
class CMPManager private constructor(
    private val context: Context,
    private val urlConfig: UrlConfig,
    private val webViewConfig: ConsentLayerUIConfig,
    private val delegate: CMPManagerDelegate,
    private var onClickLinkCallback: OnClickLinkCallback? = null
) : WebViewManagerDelegate, NetworkMonitorDelegate,
    DefaultLifecycleObserver {

    companion object {
        private val instanceRef: AtomicReference<CMPManager?> = AtomicReference(null)

        fun getInstance(
            context: Context,
            urlConfig: UrlConfig,
            webViewConfig: ConsentLayerUIConfig = ConsentLayerUIConfig(),
            delegate: CMPManagerDelegate
        ): CMPManager {
            return instanceRef.get() ?: synchronized(this) {
                instanceRef.get() ?: CMPManager(
                    context.applicationContext,
                    urlConfig,
                    webViewConfig,
                    delegate
                ).also {
                    instanceRef.set(it)
                }
            }
        }
    }

    private val coroutineScope = CoroutineScope(Dispatchers.Main)
    private val userPreferences: CMPUserPreferencesService = CMPUserPreferencesServiceImpl(context)
    private val webViewManager: WebViewManager
    private val networkMonitor: NetworkMonitor = NetworkMonitor(context)
    private val retryManager: RetryManager

    private var activityContext: WeakReference<Activity>? = null

    private var consentLayerDialog: ConsentLayerDialog? = null
    private var webView: WebView? = null
    private var webViewContainer: WebViewContainer? = null

    init {
        networkMonitor.delegate = this

        webViewManager = WebViewManager(userPreferences, coroutineScope, networkMonitor)
        webViewManager.delegate = this

        retryManager = RetryManager(maxRetryCount = 3, baseDelay = 2000L, retryable = webViewManager)

        setupWebView()
    }

    // region UI/UX setup
    fun setActivity(activity: Activity) {
        activityContext = WeakReference(activity)
        setupWebView()
    }

    fun setWebView(webView: WebView) {
        this.webView = webView
        setupWebView()
    }

    private fun setupWebView() {
        val activity = activityContext?.get() ?: return

        webView = webViewManager.createWebView(WeakReference(activity))
        webViewContainer = webView?.let { WebViewContainer(it) }
    }

    // endregion

    // region Private methods

    private fun cleanUp() {
        activityContext?.clear()
        dismissConsentLayer()
        webViewContainer = null
    }

    private fun loadConsentURLWithUseCase(useCase: UseCase, additionalParams: Map<String, Any>? = null, completion: (Result<Boolean>) -> Unit) {
        val params = createUrlParams(useCase)
        additionalParams?.let { params.apply(it) }

        val urlString = CMPUrlBuilder.build(params)
        println("Debug: URL for $useCase - $urlString") // Add this line for debugging

        webViewManager.loadConsentURL(urlString, useCase) { result ->
            completion(result)
        }
    }

    private fun showConsentLayer() {
        Log.d("CMPManager", "Showing consent layer")
        val activity = activityContext ?: run {
            delegate.didReceiveError("No valid activity context available")
            return
        }
        val config = webViewConfig

        if (webView == null) {
            try {
                webView = webViewManager.createWebView(activity)
                webViewContainer = WebViewContainer(webView!!)
            } catch (e: Exception) {
                Log.e("CMPManager", "Failed to create WebView", e)
                return
            }
        }

        if (consentLayerDialog == null || consentLayerDialog?.isShowing() == false) {
            Log.d("CMPManager", "Creating new consent layer dialog")
            consentLayerDialog?.dismiss()
            consentLayerDialog = ConsentLayerDialog(activity.get()!!, config, webViewContainer!!, webViewManager)
            consentLayerDialog?.show()
        }
        delegate.didShowConsentLayer()
    }

    private fun dismissConsentLayer() {
        Log.d("CMPManager", "Dismissing consent layer")

        consentLayerDialog?.dismiss()
        consentLayerDialog = null
    }

    private fun createUrlParams(useCase: UseCase): CMPUrlParams {
        return CMPUrlParams(
            id = urlConfig.id,
            domain = urlConfig.domain,
            useCase = useCase,
            consent = getConsentModel()?.exportCMPInfo(),
            language = urlConfig.language,
            appName = urlConfig.appName,
            packageName = context.packageName,
            isDebugMode = false
        )
    }

    private fun getConsentModel(): CMPConsentModel? {
        val (jsonString, _) = userPreferences.getConsentData()
        return jsonString?.let { CMPConsentModel.fromJson(it) }
    }
    // endregion

    // region Methods handling persisted data
    fun getUserStatus(): UserConsentStatus {
        return getConsentModel()?.toUserConsentStatus() ?: UserConsentStatus(
            hasUserChoice = UserChoiceStatus.CHOICE_DOESNT_EXIST,
            vendors = emptyMap(),
            purposes = emptyMap(),
            tcf = "",
            addtlConsent = "",
            regulation = ""
        )
    }

    fun getStatusForPurpose(id: String): ConsentStatus {
        return getConsentModel()?.getStatusForPurpose(id) ?: ConsentStatus.CHOICE_DOESNT_EXIST
    }

    fun getStatusForVendor(id: String): ConsentStatus {
        return getConsentModel()?.getStatusForVendor(id) ?: ConsentStatus.CHOICE_DOESNT_EXIST
    }

    fun exportCMPInfo(): String = getConsentModel()?.exportCMPInfo() ?: ""

    fun resetConsentManagementData(): Unit = userPreferences.resetConsentManagementData()

    // Add this method to the CMPManager class in CMPManager.kt

    /**
     * Returns a Map of consent settings compatible with Google Consent Mode v2.
     * This map can be directly passed to Firebase Analytics' setConsent method.
     *
     * @return Map<String, String> where keys are consent settings (analytics_storage, ad_storage, etc.)
     *         and values are "granted" or "denied"
     */
    fun getGoogleConsentModeStatus(): Map<String, String> {
        Log.d("CMPManager", "Generating Google Consent Mode v2 settings")

        val consentModel = getConsentModel()
        val consentModeSettings = consentModel?.consentMode

        // If no consent model or settings exist, default all to denied as per Google's recommendations
        if (consentModel == null || consentModeSettings == null) {
            Log.w("CMPManager", "No consent settings found, defaulting all to denied")
            return ConsentType.entries.associate {
                it.name.lowercase() to "denied"
            }
        }

        return consentModeSettings.map { (type, status) ->
            type.name.lowercase() to when (status) {
                ConsentStatus.GRANTED -> "granted"
                else -> "denied"
            }
        }.toMap().also {
            Log.d("CMPManager", "Generated consent settings: $it")
        }
    }

    // endregion

    // region WebView Methods
    fun checkAndOpen(jumpToSettings: Boolean = false, completion: (Result<Unit>) -> Unit) {
        if (activityContext?.get() == null) {
            completion(Result.failure(IllegalStateException("Activity reference is null. Call setActivity() before checking consent.")))
            return
        }

        loadConsentURLWithUseCase(UseCase.VERIFY_CONSENT_ON_INITIALIZE, additionalParams = mapOf("jumpToSettings" to jumpToSettings)) { result ->
            result.onSuccess { needsConsent ->
                if (needsConsent) {
                    showConsentLayer()
                } else {
                    delegate.didCloseConsentLayer()
                }
                completion(Result.success(Unit))
            }.onFailure { error ->
                completion(Result.failure(error))
            }
        }
    }

    suspend fun checkAndOpen(jumpToSettings: Boolean = false): Result<Unit> = suspendCancellableCoroutine { continuation ->
        checkAndOpen(jumpToSettings) { result ->
            continuation.resumeWith(kotlin.Result.success(result))
        }
    }

    fun forceOpen(jumpToSettings: Boolean = false, completion: (Result<Unit>) -> Unit) {
        Log.d("CMPManager", "Force opening consent layer")

        webView = activityContext?.let { webViewManager.createWebView(it) }
        webViewContainer = WebViewContainer(webView!!)

        loadConsentURLWithUseCase(UseCase.OPEN_CONSENT, mapOf("jumpToSettings" to jumpToSettings)) { result ->
            result.onSuccess { needsConsent ->
                if (needsConsent) {
                    showConsentLayer()
                } else {
                    delegate.didCloseConsentLayer()
                }
                completion(Result.success(Unit))
            }.onFailure { error ->
                completion(Result.failure(error))
            }
        }
    }

    fun importCMPInfo(cmpString: String, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.IMPORT_CONSENT, mapOf("consent" to cmpString)) { result ->
            completion(result.map {  })
        }
    }

    fun acceptVendors(vendors: List<String>, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.ENABLE_CONSENT_VENDORS, mapOf("addVendors" to vendors)) { result ->
            completion(result.map {  })
        }
    }

    fun rejectVendors(vendors: List<String>, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.DISABLE_CONSENT_VENDORS, mapOf("addVendors" to vendors)) { result ->
            completion(result.map {  })
        }
    }

    fun acceptPurposes(purposes: List<String>, updatePurpose: Boolean, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.ENABLE_CONSENT_PURPOSES, mapOf("addPurposes" to purposes, "updateVendors" to updatePurpose)) { result ->
            completion(result.map {  })
        }
    }

    fun rejectPurposes(purposes: List<String>, updateVendor: Boolean, completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.DISABLE_CONSENT_PURPOSES, mapOf("addPurposes" to purposes, "updateVendors" to updateVendor)) { result ->
            completion(result.map {  })
        }
    }

    fun rejectAll(completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.REJECT_ALL_CONSENT, mapOf("rejectAll" to true)) { result ->
            completion(result.map {  })
        }
    }

    fun acceptAll(completion: (Result<Unit>) -> Unit) {
        loadConsentURLWithUseCase(UseCase.ACCEPT_ALL_CONSENT, mapOf("acceptAll" to true)) { result ->
            completion(result.map {  })
        }
    }
    // endregion

    // region Delegate methods
    fun onApplicationPause() {
        webViewManager.cancelOperations()
        Log.d("CMPManager", "Application paused")
    }

    fun onApplicationResume() {
        Log.d("CMPManager", "Application resumed")
    }

    override fun didReceiveConsentMessage(consent: String, jsonObject: JsonObject) {
        Log.d("CMPManager", "Consent message received")
        delegate.didReceiveConsent(consent, jsonObject)
        dismissConsentLayer()
    }

    override fun didReceiveOpenMessage() {
        Log.d("CMPManager", "Open message received")
        coroutineScope.launch(Dispatchers.Main) {
            showConsentLayer()
        }
        delegate.didShowConsentLayer()
    }

    override fun didNotNeedOpenConsentLayer() {
        Log.d("CMPManager", "Close message received")
    }

    override fun didReceiveError(error: String) {
        Log.e("CMPManager", "Error received: $error")
        delegate.didReceiveError(error)
    }

    override fun networkStatusDidChange(isConnected: Boolean) {
        if (isConnected) {
            // Retry any pending operations if the network is back
            retryManager.retry { /* Handle retry result if needed */ }
        }
    }

    override fun onDestroy(owner: LifecycleOwner) {
        Log.d("CMPManager", "Activity is being destroyed")
        if (owner.lifecycle.currentState.isAtLeast(Lifecycle.State.RESUMED)) {
            Log.d("CMPManager", "Configuration is changing, not cleaning up")
            return
        }
        cleanUp()
    }

    override fun onResume(owner: LifecycleOwner) {
        Log.d("CMPManager", "Activity resumed")
        // Reinitialize WebView if needed
        val activity = activityContext
        if (activity != null && webViewManager.isWebViewDestroyed()) {
            val webView = webViewManager.createWebView(activity)
            webViewContainer = webView?.let { WebViewContainer(it) }
        }
    }

    fun onActivityDestroyed() {
        activityContext = null
        dismissConsentLayer()
        webView = null
        webViewContainer = null
    }

    fun setOnClickLinkCallback(callback: OnClickLinkCallback?) {
        onClickLinkCallback = callback
    }

    override fun getOnClickLinkCallback(): OnClickLinkCallback? = onClickLinkCallback

    // endregion

    // region Deprecated Methods to be removed
    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun checkIfConsentIsRequired(completion: (Boolean) -> Unit) {
        loadConsentURLWithUseCase(UseCase.PERFORM_DRY_CHECK_CONSENT) { result ->
            result.onSuccess { needsConsent ->
                completion(needsConsent)
            }.onFailure {
                completion(true)
            }
        }
    }

    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun hasUserChoice(): Boolean = getConsentModel()?.hasUserChoice() ?: false

    @Deprecated(
        message = "Use getStatusForPurpose() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getStatusForPurpose(id: String)")
    )
    fun hasPurposeConsent(id: String): Boolean = getConsentModel()?.hasPurposeConsent(id) ?: false
    @Deprecated(
        message = "Use getStatusForVendor() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getStatusForVendor(id: String)")
    )
    fun hasVendorConsent(id: String): Boolean = getConsentModel()?.hasVendorConsent(id) ?: false

    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun getAllPurposesIDs(): List<String> = getConsentModel()?.getAllPurposesIDs() ?: emptyList()
    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun getEnabledPurposesIDs(): List<String> = getConsentModel()?.getEnabledPurposesIDs() ?: emptyList()
    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun getDisabledPurposesIDs(): List<String> = getConsentModel()?.getDisabledPurposesIDs() ?: emptyList()
    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun getAllVendorsIDs(): List<String> = getConsentModel()?.getAllVendorsIDs() ?: emptyList()
    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun getEnabledVendorsIDs(): List<String> = getConsentModel()?.getEnabledVendorsIDs() ?: emptyList()
    @Deprecated(
        message = "Use getUserStatus() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("getUserStatus()")
    )
    fun getDisabledVendorsIDs(): List<String> = getConsentModel()?.getDisabledVendorsIDs() ?: emptyList()

    @Deprecated(
        message = "Use forceOpen() instead. This method will be removed in the next version.",
        replaceWith = ReplaceWith("forceOpen()")
    )
    fun openConsentLayer(completion: (Result<Unit>) -> Unit) {
        Log.d("CMPManager", "Force opening consent layer")

        webView = activityContext?.let { webViewManager.createWebView(it) }
        webViewContainer = WebViewContainer(webView!!)

        loadConsentURLWithUseCase(UseCase.OPEN_CONSENT) { result ->
            result.onSuccess {
                showConsentLayer()
                delegate.didShowConsentLayer()
                completion(Result.success(Unit))
            }.onFailure { error ->
                Log.e("CMPManager", "Error in forceOpenConsentLayer: ${error.message}")
                completion(Result.failure(error))
            }
        }
    }

    @Deprecated(
        message = "Use checkAndOpen() instead",
        replaceWith = ReplaceWith("checkAndOpen(false, completion)")
    )
    fun checkWithServerAndOpenIfNecessary(completion: (Result<Unit>) -> Unit) {
        if (activityContext?.get() == null) {
            completion(Result.failure(IllegalStateException("Activity reference is null. Call setActivity() before checking consent.")))
            return
        }

        loadConsentURLWithUseCase(UseCase.VERIFY_CONSENT_ON_INITIALIZE) { result ->
            result.onSuccess { needsConsent ->
                if (needsConsent) {
                    showConsentLayer()
                } else {
                    delegate.didCloseConsentLayer()
                }
                completion(Result.success(Unit))
            }.onFailure { error ->
                completion(Result.failure(error))
            }
        }
    }

    // endregion
}
